#***********************************************************************

# Licensed Material - Property of IBM

#

# 5724-M24

#

# (c) Copyright IBM Corp. 2008.  All rights reserved.

#

# US Government Users Restricted Rights - Use, duplication or

# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

#************************************************************************



#Sample command.

#wsadmin.bat -user admin -password admin -f fabricSIBNode.py -node localhostNaveenNode04 -toolsUserId admin --fabricUTECreateNew true



##################################################################

# Creates the bus and deploy fabric applications

# for the Websphere Business Services Fabric on Node.

# 

# wsadmin <wsadmin options> -f fabricSIBNode.py    

#                          [-node nodeName]

#			   [-server serverName] 

#                          [-toolsUserId toolsUserId]

#			   [-fabricUTECreateNew fabricUTECreateNew] 

#

# 

# If there is only one server in your node you don't need to supply the server name.

# In standalone profile you would have only one server.

# 

# The -fabricUTECreateNew = true would create ute variables.

# 

#############################################################



import sys

import java





##########################################################

# Prints usage error 

###########################################################



def usageError(error):

    sys.stderr.write(error)

    print "Exception:Incorrect usage" + str(error)

    sys.exit()



###########################################################

# Prints error message on the console

###########################################################

def validationError (errorMessage) :

    sys.stderr.write(errorMessage)

    print "Exception:validationError" + str(errorMessage)

    raise



##########################################################

#Get websphere variables based on scope id : Cell id /Node id /Server id /Cluster id

##########################################################

def getWebsphereVariable(variablename,scope) :

    variableEntry = AdminConfig.list("VariableSubstitutionEntry",scope).splitlines()

    value = "";

    for ventry in variableEntry :

        name = AdminConfig.showAttribute(ventry,"symbolicName")

        if(name == variablename):

            value = AdminConfig.showAttribute(ventry,"value")

            break

    return value



##########################################################

#Get Datasources based on JNDIname and scope.

##########################################################

def findDataSource(jndiName,scope):



    dataSourceList = AdminConfig.list("DataSource", scope).splitlines() 

    for dsentry in dataSourceList :

	dsJndiName = AdminConfig.showAttribute(dsentry,"jndiName")

	if(dsJndiName == jndiName) :

	    provider = AdminConfig.showAttribute(dsentry,"provider")

	    provideName = AdminConfig.showAttribute(provider,"name")

	    dataSourceName = AdminConfig.showAttribute(dsentry,"name")

	    path = "/Node:"+nodeName+"/Server:"+serverName+"/" + "JDBCProvider:"+ provideName + "/DataSource:" + dataSourceName+"/"

	    dataSource = AdminConfig.getid(path)

	    if(dataSource != "") :

	       return dataSource

    return ""





##########################################################

#Get JASSAuthData based on Auth Alias

##########################################################



def getJAASAuthData(aliasAuth) :

    authObj = {}

    aliasidlist = AdminConfig.list("JAASAuthData").splitlines()

    for aliasid in aliasidlist:

        qualifiedaliasName = AdminConfig.showAttribute(aliasid,"alias")

        i= qualifiedaliasName.find('/')

        name=qualifiedaliasName[i+1:len(qualifiedaliasName)]

        if (name==aliasAuth):

            authObj["userId"] = AdminConfig.showAttribute(aliasid,"userId")

            authObj["password"] = AdminConfig.showAttribute(aliasid,"password")

	    return authObj

    

    return authObj



##########################################################

#Create JAASAuthData

##########################################################



def createJAASAuthData(alias,userId,password,description):

    authObj = getJAASAuthData(alias)

    if(len(authObj) > 1):

        validationError("Auth alias" + alias + "already exists")

    else :

        security = AdminConfig.getid('/Cell:'+ cellName +'/Security:/')

        aliasAttr = ['alias', alias]

        useridAttr = ['userId', userId]

        passwordAttr = ['password', password]

        descriptionAttr = ['description' , description]

        jaasAttrs = [aliasAttr, useridAttr, passwordAttr,descriptionAttr]

        #Create Auth Alias

        AdminConfig.create('JAASAuthData', security, jaasAttrs)



        

#######################################################################

#  Uninstalling the Existing Application

#######################################################################



def unInstallApplications(appList):

    installedApps = AdminApp.list().splitlines()    

    try:

        for app in appList:

            if(app in installedApps):

                print "Uninstalling "+app

                AdminApp.uninstall(app)

    except:

        print "Exception:Error while uninstalling "+app

        sys.stderr.write("Exception:Error while uninstalling "+app)

        raise



#-----------------------------------------------------------------------------------------------------

# update fabric rest services

#-------------------------------------------------------------------------------------------------------



def updateRestServices(restServiceTypes, nodeName, serverName, clusterName):

    

    for restServiceType in restServiceTypes :

        attrs = ['-version', '6.2.0.0', '-enable', 'true']

        if clusterName == '':

            attrs.extend(["-nodeName", nodeName, "-serverName", serverName])

        else:

            attrs.extend(["-clusterName", clusterName])

        

        attrs.extend(['-type', restServiceType])

        

	print "Updating rest service: "+restServiceType

        

	AdminTask.updateRESTGatewayService(attrs)   



#------------------------------------------------------------------------------

# get the scope path according to cellname, nodeName and serveName

#------------------------------------------------------------------------------

def getScopePath(cellName, nodeName, serverName):

    scopePath = "";

    if (cellName == ''):

        scopePath = '';

    elif (nodeName == ''):

        scopePath = '/Cell:' + cellName + '/';

    elif (serverName == ''):        

        scopePath = '/Cell:' + cellName + '/Node:' + nodeName + '/';

    else:

        scopePath = '/Cell:' + cellName + '/Node:' + nodeName + '/Server:' + serverName + '/';

    #endIf

    return scopePath;

#endDef



#------------------------------------------------------------------------------

# check whether the specified application is installed 

#------------------------------------------------------------------------------

def checkApplicationExist(appName, scope):

    print "checkApplicationExist(" + appName + ") start";

    result = False

    applications = AdminApp.list(scope).split()

    for application in applications:

        if (application == appName):

            result = True

            print "checkApplicationExist() end:" + str(result);

            return result

    print "checkApplicationExist() end:" + str(result);

    return result

#endDef



#-----------------------------------------------------------------

# Check if Fabric Widgets  has been configured:

#-----------------------------------------------------------------

def installFabricWidgets(cellName,nodeName,serverName):

    

    installtionRequired = False

    installOptions = []

    

    WAS_INSTALL_DIR = java.lang.System.getProperty("was.install.root")

    widgetPath = WAS_INSTALL_DIR+"/BusinessSpace/widgets/WBSF"

    

    scopeAtt = 'WebSphere:cell=' + cellName + ',node=' + nodeName + ',server=' + serverName

    scopePath = getScopePath(cellName, nodeName, serverName)        

        

    fabric_widget_app_name  = "widgets_fabric_"+nodeName+"_"+serverName

    business_space_app_name = "BSpaceEAR_"+nodeName+"_"+serverName

    installOptions = ["-serverName",serverName,"-nodeName",nodeName]

    isBusinessSpaceDeployed = AdminTask.getBusinessSpaceDeployStatus(installOptions)

    print "isBusinessSpaceDeployed" + isBusinessSpaceDeployed

         

    installtionRequired = not checkApplicationExist(fabric_widget_app_name, scopeAtt) and (isBusinessSpaceDeployed.upper() == "TRUE")

    installOptions = ["-serverName",serverName,"-nodeName",nodeName,"-widgets",widgetPath,"-save false"] 

        

    if(installtionRequired):

	print "Installing Fabric widgets"

        AdminTask.installBusinessSpaceWidgets(installOptions)	



############################################################

# Variables

############################################################



#Constants for Boolean types

True = 1==1

False = 1==0

isVMMConfigured = True



installableApps = '../../../installableApps/'

#Conditional variables

doMEDatastore=False

doMEInputFilestore=False

doMEInputDataStore=False

doMEInputFilestoreValue = ""

doMEInputDataStoreValue = ""

createDerbyDBForME=False

createTables=True

doFabricUTECreateNew=""



#Config parameters

nodeName=""

serverName=""

fabricBusPassword=""

cellName=""

cellId=""

fabricBusName=""

fabricJNDIMEName=""





#SIB Bus details

scaBusCellName=""

scaSystemBus=""

sibEngine=""

scaDataStoreJNDIName=""

toolsUserId=""



# Data structure for holding SCA Bus/ME properties

fabricSCAMEProperty ={}

fabricSCAMEProperty["resources"] = ""

fabricSCAMEProperty["datasourceHelperClassname"] = ""

fabricSCAMEProperty["provider"] = ""

fabricBusProperty={}



#Constants

fabricMEAuthAlias= "Fabric_ME_Auth_Alias"

fabricBusAuthAlias = "Fabric_Bus_Auth_Alias"

fabricSchemaName = "FABRICME"

meDatabaseName ="FABRICME"

lineSeparator = java.lang.System.getProperty('line.separator')





###################################################################

# Check if the script is being run interactively

###################################################################



if ((len(sys.argv) == 0)):

    validationError("No inputs provided")

        

###################################################################

# Command line parameters passed in. Evaluate each of these paramters

###################################################################

else:



    try :

    

	    nextArg = 0;

	  

	    while(len(sys.argv) > nextArg) :

		param = sys.argv[nextArg]

		nextArg +=1



		if (len(sys.argv) <= nextArg):

		    usageError("specify value for" + param  + " parameter")



		value = sys.argv[nextArg]

		nextArg +=1



		if (param == "-node"):

		    nodeName = value

		elif (param == "-server"):

		    serverName = value

		elif (param == "-fabricUTECreateNew"):

		    doFabricUTECreateNew = value

		elif (param == "-toolsUserId"): 

		    toolsUserId = value

		else :

		    usageError("Unknown parameter " + param)



	###################################################################

	# Validate paramter values

	###################################################################

	    if(len(nodeName) < 1 ):

		usageError("NodeName not provided")

	    

	    nodeId=AdminConfig.getid("/Node:"+nodeName+"/")

	    if (nodeId==""):

		validationError("Node "+ nodeName + "doesn't exist")

	    

	    if(len(serverName) < 1):

		servers = AdminConfig.list("Server", nodeId)

		serversList = servers.split(lineSeparator)

		serverName =  AdminConfig.showAttribute(serversList[0], "name")

	    

	    serverId=AdminConfig.getid("/Node:"+nodeName+"/Server:"+serverName+"/")

	    if (serverId==""):

		validationError("Server "+ serverName + " doesn't exist")

	    

	    

	    celllist = AdminConfig.list("Cell").splitlines()

	    cellId = celllist[0]

	    i= cellId.find('(')

	    cellName=cellId[0:i]



	    if(cellId == ""):

		validationError("cell " + cellName + " doesn't exist")



	    if(len(toolsUserId) < 1):

		validationError("-toolsUserId value not provided for mapping user to Tools ear.")

	    

	    if(len(doFabricUTECreateNew) < 1):

		validationError("-fabricUTECreateNew value not provided.Please provide a value.")

	    

	###############################################################

	# End of validation

	###################################################################



	###################################################################

	# If script is being run interactively, query for values.

	###################################################################

	    ## Set UTE Variables



	    try :

		if(doFabricUTECreateNew.upper() == "TRUE") :

		    attr1 = ["name","wbsf.gov.skip"]

		    attr2 = ["nameInNameSpace","wbsf.gov.skip"]

		    attr3 = ["stringToBind","true"]

		    options = []

		    options.append(attr1)

		    options.append(attr2)

		    options.append(attr3)

		    AdminConfig.create("StringNameSpaceBinding",cellId,options)

		    print "INFO - Set the wbsf.gov.skip=true"



		    attr1 = ["name","wbsf.environment"]

		    attr2 = ["nameInNameSpace","wbsf.environment"]

		    attr3 = ["stringToBind","ute"]

		    options = []

		    options.append(attr1)

		    options.append(attr2)

		    options.append(attr3)

		    AdminConfig.create("StringNameSpaceBinding",cellId,options)

		    print "INFO - Set the wbsf.environment=true"



	    except :

		print "Failed to set wbsf.environment variables.You need to manually set the  wbsf.environment variable."





	    try :



		security = AdminConfig.getid('/Cell:'+ cellName +'/Security:/')

		activeRegistry = AdminConfig.showAttribute(security,"activeUserRegistry")



		#You will not have the property registryClassName for other repository providers.

		try :

		    AdminConfig.showAttribute(activeRegistry,"registryClassName")

		except:

		    isVMMConfigured = False

		    print "INFO - The profile doesn't uses VMM,hence not setting variable useVMMForUsers"



		if(isVMMConfigured):

		    attr1 = ["name","useVMMForUsers"]

		    attr2 = ["nameInNameSpace","useVMMForUsers"]

		    attr3 = ["stringToBind","true"]

		    options = []

		    options.append(attr1)

		    options.append(attr2)

		    options.append(attr3)

		    AdminConfig.create("StringNameSpaceBinding",cellId,options)

		    print "INFO - Set the useVMMForUsers=true"

	    except :

		print "Failed to set useVMMForUsers.You need to manually set the useVMMForUsers variable."



	    #Deploying Fabric Ears 	

	    try :

	        appOptions = ["-appname", "Fabric_Catalog", "-server", serverName]

		AdminApp.install(installableApps+"/catalog.ear", appOptions)

	    except :

		print "Exception:Error occured while deploying Catalog ear"

		sys.stderr.write("Error occured while deploying Catalog ear")

		raise

	    print "Deployed catalog ear."



#	    try :

#	        appOptions = ["-appname", "Fabric_Tools_Help", "-server", serverName]

#		AdminApp.install(installableApps+"/tools-help.ear", appOptions)

#	    except :

#		print "Exception:Error occured while deploying Tools-Help ear"

#		sys.stderr.write("Error occured while deploying Tools-Help ear")

#		raise

#	    print "Deployed tools-help ear."

	    try :

		nameOpt = ["-appname", "Fabric_Tools"]

		serverOpt = ["-server", serverName]

		mapUserValue = [["FabricAdministrator", "No", "No" , toolsUserId , ""]]

		mapUserOpt = ["-MapRolesToUsers", mapUserValue] 

		appOptions = []

		appOptions.extend(nameOpt)

		appOptions.extend(mapUserOpt)

		appOptions.extend(serverOpt)

		AdminApp.install(installableApps+"/tools.ear",appOptions)

	    except :

	        print "Exception:Error occured while deploying tools ear"

		sys.stderr.write("Error occured while deploying tools ear")

		raise 

	    print "Deployed Tools ear."

	    try :

	        appOptions = ["-appname", "Fabric_Trace", "-server", serverName]

		AdminApp.install(installableApps+"/trace.ear", appOptions)

	    except :

		print "Exception:Error occured while deploying Trace ear"

		sys.stderr.write("Error occured while deploying Trace ear")

		raise

	    print "Deployed trace ear."

	    

	    #Set the DA Event Tracer.

	    try:

		attr1 = ["name","wbsf.da.executionTrace.viewer.enable"]

		attr2 = ["nameInNameSpace","wbsf.da.executionTrace.viewer.enable"]

		attr3 = ["stringToBind","true"]



		options = []

		options.append(attr1)

		options.append(attr2)

		options.append(attr3)

		AdminConfig.create("StringNameSpaceBinding",cellId,options)





		attr1 = ["name","wbsf.da.executionTrace.maxcount"]

		attr2 = ["nameInNameSpace","wbsf.da.executionTrace.maxcount"]

		attr3 = ["stringToBind","100"]



		options = []

		options.append(attr1)

		options.append(attr2)

		options.append(attr3)

		AdminConfig.create("StringNameSpaceBinding",cellId,options)

	    except :

		print "Failed to set DA execution variables.You need to manually set the wbsf.da.executionTrace.viewer.enable and wbsf.da.executionTrace.maxcount variable."





	    #Create the cache provider

	    #Create the Fabric Context Cache

	    try :

		cacheProvider = AdminConfig.getid("/Node:"+nodeName+"/CacheProvider:CacheProvider/")

		

		cacheOption1 = ["name", "Fabric Context Cache"]

		cacheOption2 = ["jndiName", "services/cache/wbsf.contexts"]

		cacheOptions = []

		cacheOptions.append(cacheOption1)

		cacheOptions.append(cacheOption2)

		AdminConfig.create("ObjectCacheInstance",cacheProvider,cacheOptions)



		cacheOption1 = ["name", "Fabric Mediation Endpoint Selection Cache"]

		cacheOption2 = ["jndiName", "services/cache/fabric/mediation/selection"]

		cacheOption3 = ["cacheSize", "1000"]

		cacheOptions = []

		cacheOptions.append(cacheOption1)

		cacheOptions.append(cacheOption2)

		cacheOptions.append(cacheOption3)

		AdminConfig.create("ObjectCacheInstance",cacheProvider,cacheOptions)



		cacheOption1 = ["name", "Fabric Mediation Interface Endpoints Cache"]

		cacheOption2 = ["jndiName", "services/cache/fabric/mediation/endpoint"]

		cacheOption3 = ["cacheSize", "100"]

		cacheOptions = []

		cacheOptions.append(cacheOption1)

		cacheOptions.append(cacheOption2)

		cacheOptions.append(cacheOption3)

		AdminConfig.create("ObjectCacheInstance",cacheProvider,cacheOptions)



		cacheOption1 = ["name", "Fabric Mediation Subscription Cache"]

		cacheOption2 = ["jndiName", "services/cache/fabric/mediation/subscription"]

		cacheOptions = []

		cacheOptions.append(cacheOption1)

		cacheOptions.append(cacheOption2)

		AdminConfig.create("ObjectCacheInstance",cacheProvider,cacheOptions)



	    except :

		print "Failed to set Fabric context cache.You need to manually set the Fabric context cache."

	    

	    try :

		restServiceTypes = ['{com.ibm.bpm}BSLM', '{com.ibm.bpm}Glossary', '{com.ibm.bpm}Governance','{com.ibm.bpm}BVARS']

		updateRestServices(restServiceTypes, nodeName, serverName, '')

		print "Successfully updated fabric rest services."

	    except:

		print "Failed to update fabric rest services."

	    

	    try :

		AdminTask.registerRESTServiceEndpoint(['-businessSpaceNodeName', nodeName, '-businessSpaceServerName', serverName])            

		print "Successfully registered fabric rest services."

   	    except:

		print "Failed to register fabric rest services with business space."

	    

	    try :

                installFabricWidgets(cellName,nodeName,serverName)

	    except :

	        print " Failed to Install Fabric widgets. Please install it manually."

	    

	    #Save configuration

	    AdminConfig.save()



    except :

        print "Exception:" + str(sys.exc_info())

        sys.exit()

      

########################################################################################